import 'dart:async';
import 'package:meta/meta.dart';
import 'package:flutter/services.dart';
import 'package:interactive_webview/interactive_webview.dart';
import 'package:uuid/uuid.dart';
import 'package:sentry/sentry.dart';
import 'message_broker.dart';
import 'dart:convert';
import 'dto.dart';
import 'player/tivio_player_wrapper.dart';
import 'dto_parsers/tivio_data_widget_parser.dart';
import 'dto_parsers/tivio_data_channel_parser.dart';
import 'dto_parsers/tivio_data_section_parser.dart';
import 'dto_parsers/tivio_data_video_parser.dart';
import 'streams.dart';
import 'logger.dart';

class Bridge {
  static final Bridge _instance = Bridge._internal();
  final _uuid = Uuid();
  final _webView = InteractiveWebView();
  final _jsonEncoder = JsonEncoder();
  final _dataMessageBroker = DataMessageBroker();
  final _callbackMessageBroker = CallbackMessageBroker();
  final _logger = Logger();
  Completer _bridgeReady;

  factory Bridge() {
    return _instance;
  }

  Bridge._internal();

  loadBridgeJs() {
    PlatformAssetBundle loader = PlatformAssetBundle();
    return loader.loadString('packages/tivio_sdk/assets/bridge.build.js');
  }

  Future<void> init(String appId, String language, bool development) async {
    _logger.debug('Tivio bridge: initialization started');

    if (_bridgeReady != null) {
      _logger.debug('Tivio bridge: waiting for completion of initialization');
      return _bridgeReady.future;
    }

    try {
      _bridgeReady = Completer();
      _logger.debug('Tivio bridge: loading of js build started');
      String bundle = await loadBridgeJs();
      _logger.debug('Tivio bridge: loading of js build completed. Bundle size: ${bundle.length}');

      _webView.didReceiveMessage.listen((event) {
        switch (event.data['type']) {
          case 'data':
            _dataMessageBroker.publish(
              event.data['messageId'],
              event.data['data'],
            );
            break;
          case 'callback':
            _callbackMessageBroker.publish(
              event.data['messageId'],
              event.data['data'],
            );
            break;
        }
      });

      _webView.stateChanged.listen((event) {
        if (event.type == WebViewState.didFinish) {
          callFunction(
            'init',
            [appId, language, development],
            (payload) {
              _logger.debug('Tivio bridge: initialization completed');
              _bridgeReady.complete();
            },
          );
        }
      });

      _webView.loadHTML('''
<!DOCTYPE html>
<html>
  <head>
    <meta charset="UTF-8">
    <script>
      $bundle
    </script>
  </head>
  <body>
  </body>
</html>
''', baseUrl: 'https://tiv.io');

      _logger.debug('Tivio bridge: initialization ended');
      return _bridgeReady.future;
    } catch (exception, stackTrace) {
      await Sentry.captureException(
        exception,
        stackTrace: stackTrace,
      );
      throw exception;
    }
  }

  getPaginatedData<T>(
    String functionName,
    List<dynamic> functionArgs,
    T Function(dynamic record) createDto,
  ) {
    return callFunction<TivioDataPagination<T>>(
      functionName,
      functionArgs,
      (payload) {
        return TivioDataPagination<T>(
          payload['records'].map<T>((record) => createDto(record)).toList(),
          payload['hasNextPage'] ?? false,
        );
      },
    );
  }

  Future<T> callFunction<T>(String functionName, List<dynamic> functionArgs, T Function(dynamic payload) onComplete) {
    Completer done = new Completer<T>();

    String messageId = _uuid.v4();
    functionArgs.add(messageId);
    String serializedArgs = _jsonEncoder.convert(functionArgs);

    _webView.evalJavascript('''window.TivioBridge.$functionName($serializedArgs);''');

    _dataMessageBroker.listen(messageId, (payload) => done.complete(onComplete(payload)));

    return done.future;
  }

  setLanguage(String language) {
    return callFunction('setLanguage', [language], (payload) => null);
  }

  /// Player

  getPlayer(String id) {
    return callFunction<TivioPlayerWrapper>(
      'getPlayer',
      [id],
      (payload) => TivioPlayerWrapper(id, this),
    );
  }

  playerRegister(String id) {
    return callFunction('playerRegister', [id], (payload) => null);
  }

  playerOnSourceChanged(String id, data) {
    return callFunction(
      'playerOnSourceChanged',
      [id, _jsonEncoder.convert(data)],
      (payload) => null,
    );
  }

  playerPlay(String id) {
    return callFunction('playerPlay', [id], (payload) => {});
  }

  playerPause(String id) {
    return callFunction('playerPause', [id], (payload) => {});
  }

  playerSeekTo(String id, num timestampMs) {
    return callFunction('playerSeekTo', [id, timestampMs], (payload) => {});
  }

  playerOnTimeChanged(String id, num timestampMs) {
    return callFunction(
      'playerOnTimeChanged',
      [id, timestampMs],
      (payload) => null,
    );
  }

  playerOnStateChanged(String id, String state) {
    return callFunction('playerOnStateChanged', [id, state], (payload) => null);
  }

  onPlaybackEnded(String id) {
    return callFunction('playerOnPlaybackEnded', [id], (payload) => null);
  }

  playerSkipAd(String playerId) {
    return callFunction('playerSkipAd', [playerId], (payload) => null);
  }

  onPlayerPlay(String id, Function callback) {
    _callbackMessageBroker.listen('player-$id-play', callback);
  }

  onPlayerPause(String id, Function callback) {
    _callbackMessageBroker.listen('player-$id-pause', callback);
  }

  onPlayerChangeSource(String id, Function callback) {
    _callbackMessageBroker.listen('player-$id-set-source', callback);
  }

  onPlayerSeekTo(String id, Function callback) {
    _callbackMessageBroker.listen('player-$id-seek-to', callback);
  }

  /// API 1.0.0

  /// Widget
  TivioDataStream<TivioDataWidget> subscribeToWidget({
    @required String widgetId,
  }) {
    var controller = TivioDataStreamController<TivioDataWidget>();

    _callbackMessageBroker.listen('subscribe-to-widget:$widgetId', (payload) {
      if (payload['error'] != null) {
        _logger.error(payload['error']);
        controller.addErrorSafely(ArgumentError(payload['error']['message']));
      } else {
        controller.addSafely(TivioDataWidgetParser.parseWidget(payload));
      }
    });

    callFunction('subscribeToWidget', [widgetId], (payload) => null);

    return controller.tivioDataStream;
  }

  /// Channel

  TivioDataStream<TivioDataPagination<TivioDataChannel>> subscribeToChannelsInWidget({
    @required String widgetId,
    @required int count,
  }) {
    var controller = TivioDataStreamController<TivioDataPagination<TivioDataChannel>>();

    _callbackMessageBroker.listen('subscribe-to-channels-in-widget:$widgetId', (payload) {
      if (payload['error'] != null) {
        _logger.error(payload['error']);
        controller.addErrorSafely(ArgumentError(payload['error']['message']));
      } else {
        controller.addSafely(TivioDataChannelParser.parsePaginatedChannels(payload));
      }
    });

    callFunction('subscribeToChannelsInWidget', [widgetId, count], (payload) => null);

    return controller.tivioDataStream;
  }

  fetchNextChannelsInWidget({
    @required String widgetId,
    @required int count,
  }) {
    callFunction('fetchNextChannelsInWidget', [widgetId, count], (payload) => null);
  }

  TivioDataStream<TivioDataChannel> subscribeToChannel({@required String channelId}) {
    var controller = TivioDataStreamController<TivioDataChannel>();

    _callbackMessageBroker.listen('subscribe-to-channel:$channelId', (payload) {
      if (payload['error'] != null) {
        _logger.error(payload['error']);
        controller.addErrorSafely(ArgumentError(payload['error']['message']));
      } else {
        controller.addSafely(TivioDataChannelParser.parseChannel(payload));
      }
    });

    callFunction('subscribeToChannel', [channelId], (payload) => null);

    return controller.tivioDataStream;
  }

  /// Section

  TivioDataStream<TivioDataPagination<TivioDataSection>> subscribeToSectionsInChannel({
    @required String channelId,
    @required int count,
  }) {
    var controller = TivioDataStreamController<TivioDataPagination<TivioDataSection>>();

    _callbackMessageBroker.listen('subscribe-to-sections-in-channel:$channelId', (payload) {
      if (payload['error'] != null) {
        _logger.error(payload['error']);
        controller.addErrorSafely(ArgumentError(payload['error']['message']));
      } else {
        controller.addSafely(TivioDataSectionParser.parsePaginatedSections(payload));
      }
    });

    callFunction('subscribeToSectionsInChannel', [channelId, count], (payload) => null);

    return controller.tivioDataStream;
  }

  fetchNextSectionsInChannel({
    @required String channelId,
    @required int count,
  }) {
    callFunction('fetchNextSectionsInChannel', [channelId, count], (payload) => null);
  }

  TivioDataStream<TivioDataSection> subscribeToSection({@required String sectionId}) {
    var controller = TivioDataStreamController<TivioDataSection>();

    _callbackMessageBroker.listen('subscribe-to-section:$sectionId', (payload) {
      if (payload['error'] != null) {
        _logger.error(payload['error']);
        controller.addErrorSafely(ArgumentError(payload['error']['message']));
      } else {
        controller.addSafely(TivioDataSectionParser.parseSection(payload));
      }
    });

    callFunction('subscribeToSection', [sectionId], (payload) => null);

    return controller.tivioDataStream;
  }

  /// Videos

  TivioDataStream<TivioDataPagination<TivioDataVideo>> subscribeToVideosInSection({
    @required String sectionId,
    @required int count,
  }) {
    var controller = TivioDataStreamController<TivioDataPagination<TivioDataVideo>>();

    _callbackMessageBroker.listen('subscribe-to-videos-in-section:$sectionId', (payload) {
      if (payload['error'] != null) {
        _logger.error(payload['error']);
        controller.addErrorSafely(ArgumentError(payload['error']['message']));
      } else {
        controller.addSafely(TivioDataVideoParser.parsePaginatedVideos(payload));
      }
    });

    callFunction('subscribeToVideosInSection', [sectionId, count], (payload) => null);

    return controller.tivioDataStream;
  }

  fetchNextVideosInSection({
    @required String sectionId,
    @required int count,
  }) {
    callFunction('fetchNextVideosInSection', [sectionId, count], (payload) => null);
  }

  TivioDataStream<TivioDataVideo> subscribeToVideo({@required String videoId}) {
    var controller = TivioDataStreamController<TivioDataVideo>();

    _callbackMessageBroker.listen('subscribe-to-video:$videoId', (payload) {
      if (payload['error'] != null) {
        _logger.error(payload['error']);
        controller.addErrorSafely(ArgumentError(payload['error']['message']));
      } else {
        controller.addSafely(TivioDataVideoParser.parseVideo(payload));
      }
    });

    callFunction('subscribeToVideo', [videoId], (payload) => null);

    return controller.tivioDataStream;
  }

  /// Recent

  TivioDataStream<TivioDataPagination<TivioDataVideo>> subscribeToRecentVideosInWidget({
    @required String widgetId,
    @required int count,
  }) {
    var controller = TivioDataStreamController<TivioDataPagination<TivioDataVideo>>();

    controller.addSafely(TivioDataPagination([], false));

    return controller.tivioDataStream;
  }

  TivioDataStream<TivioDataPagination<TivioDataVideo>> subscribeToRecentVideosInChannel({
    @required String channelId,
    @required int count,
  }) {
    var controller = TivioDataStreamController<TivioDataPagination<TivioDataVideo>>();

    controller.addSafely(TivioDataPagination([], false));

    return controller.tivioDataStream;
  }

  /// Search

  searchChannels(String query, int limit) {
    return getPaginatedData<TivioDataChannel>(
      'searchChannels',
      [query, limit],
      (record) => TivioDataChannelParser.parseChannel(record),
    );
  }

  searchVideos(String query, int limit) {
    return getPaginatedData<TivioDataVideo>(
      'searchVideos',
      [query, limit],
      (record) => TivioDataVideoParser.parseVideo(record),
    );
  }
}
