import 'dart:async';
import 'package:flutter/material.dart';
import 'package:tivio_sdk/player/tivio_player_input_source.dart';
import 'package:better_player/better_player.dart';
import 'package:seekbar/seekbar.dart';

class TivioSeekBar extends StatefulWidget {
  final Function onSeekingStarted;
  final Function onSeekingEnded;
  final double value;

  TivioSeekBar({
    @required this.onSeekingStarted,
    @required this.onSeekingEnded,
    @required this.value,
  });

  @override
  createState() => _TivioSeekBarState();
}

class _TivioSeekBarState extends State<TivioSeekBar> {
  double internalValue;

  @override
  Widget build(BuildContext context) {
    return SeekBar(
        value: widget.value,
        thumbRadius: 9,
        progressWidth: 6,
        barColor: Color.fromRGBO(255, 255, 255, 0.2),
        progressColor: Color.fromRGBO(0, 120, 255, 1),
        thumbColor: Color.fromRGBO(0, 120, 255, 1),
        onStartTrackingTouch: () => widget.onSeekingStarted(),
        onProgressChanged: (value) => { internalValue = value },
        onStopTrackingTouch: () => widget.onSeekingEnded(internalValue));
  }
}

class TivioPlayerUiPressControls {
  final Function onPlayPressed;
  final Function onPausePressed;
  final Function onForwardPressed;
  final Function onBackwardPressed;
  final Function onEnterFullscreenPressed;
  final Function onExitFullscreenPressed;
  final Function onSeekingStarted;
  final Function onSeekingEnded;

  TivioPlayerUiPressControls({
    @required this.onPlayPressed,
    @required this.onPausePressed,
    @required this.onForwardPressed,
    @required this.onBackwardPressed,
    @required this.onEnterFullscreenPressed,
    @required this.onExitFullscreenPressed,
    @required this.onSeekingStarted,
    @required this.onSeekingEnded,
  });
}

// https://www.javatpoint.com/flutter-progress-bar
class TivioPlayerBasicUi extends StatelessWidget {
  final TivioPlayerUiPressControls controls;

  final bool isPlaying;
  final bool isInFullscreen;

  //final double progress;
  final Duration duration;
  final Duration progress;

  TivioPlayerBasicUi({
    @required this.controls,
    this.isPlaying = true,
    this.isInFullscreen = true,
    this.duration,
    this.progress,
    //this.progress = 0,
  });

  String formatNumber(int number) {
    return (number > 9) ? number.toString() : '0$number';
  }

  String createCounterLabel() {
    int restOfSeconds = (duration != null && progress != null) ? duration.inSeconds - progress.inSeconds : 0;

    Duration restAsDuration = Duration(seconds: restOfSeconds);

    return '-${formatNumber(restAsDuration.inMinutes)}:${formatNumber(restAsDuration.inSeconds % 60)}';
  }

  @override
  Widget build(BuildContext context) {
    double percentProgress = (duration != null && progress != null) ? progress.inSeconds / duration.inSeconds : 0;
    String counterLabel = createCounterLabel();

    return Positioned.fill(
      child: Padding(
        padding: EdgeInsets.all(0),
        child: Column(
          mainAxisAlignment: MainAxisAlignment.end,
          children: [
            Flexible(
              fit: FlexFit.tight,
              flex: 1,
              child: Padding(
                padding: EdgeInsets.only(top: 41),
                child: Row(
                  mainAxisAlignment: MainAxisAlignment.center,
                  children: [
                    IconButton(
                      icon: Transform(
                        alignment: Alignment.center,
                        transform: Matrix4.rotationY(3.14),
                        child: Icon(
                          Icons.fast_forward,
                          color: Colors.white,
                          size: 60,
                        ),
                      ),
                      onPressed: () => controls.onBackwardPressed(),
                      padding: EdgeInsets.all(0),
                    ),
                    IconButton(
                      icon: Icon(
                        (isPlaying) ? Icons.pause : Icons.play_arrow_outlined,
                        color: Colors.white,
                        size: 80,
                      ),
                      onPressed: () {
                        if (isPlaying) {
                          controls.onPausePressed();
                        } else {
                          controls.onPlayPressed();
                        }
                      },
                      padding: EdgeInsets.all(0),
                    ),
                    IconButton(
                      icon: Icon(
                        Icons.fast_forward,
                        color: Colors.white,
                        size: 60,
                      ),
                      onPressed: () => controls.onForwardPressed(),
                      padding: EdgeInsets.all(0),
                    ),
                  ],
                ),
              ),
            ),
            Padding(
              padding: EdgeInsets.only(left: 16, right: 16, bottom: 16),
              child: Column(
                children: [
                  /// fullscreen
                  Row(
                    mainAxisAlignment: MainAxisAlignment.end,
                    children: [
                      IconButton(
                        icon: Icon(
                          (isInFullscreen) ? Icons.close : Icons.crop_square_outlined,
                          color: Colors.white,
                          size: 60,
                        ),
                        onPressed: () {
                          print('press $isInFullscreen');
                          if (isInFullscreen) {
                            controls.onExitFullscreenPressed();
                          } else {
                            controls.onEnterFullscreenPressed();
                          }
                        },
                        padding: EdgeInsets.all(0),
                      ),
                    ],
                  ),

                  /// progress
                  Row(
                    children: [
                      /*Expanded(
                        child: Column(
                          mainAxisAlignment: MainAxisAlignment.spaceEvenly,
                          children: <Widget>[
                            LinearProgressIndicator(
                              backgroundColor: Color.fromRGBO(255, 255, 255, 0.2),
                              valueColor: AlwaysStoppedAnimation<Color>(Color.fromRGBO(0, 120, 255, 1)),
                              value: percentProgress,
                            ),
                          ],
                        ),
                      ),*/
                      /*Expanded(
                        child: SeekBar(
                            value: percentProgress,
                            thumbRadius: 9,
                            progressWidth: 6,
                            barColor: Color.fromRGBO(255, 255, 255, 0.2),
                            progressColor: Color.fromRGBO(0, 120, 255, 1),
                            thumbColor: Color.fromRGBO(0, 120, 255, 1),
                            onProgressChanged: (value) {
                              print('onProgressChanged:$value');
                            },
                            onStopTrackingTouch: () {
                              print('onStopTrackingTouch');
                            }),
                      ),*/
                      Expanded(
                        child: TivioSeekBar(
                          value: percentProgress,
                          onSeekingStarted: () => controls.onSeekingStarted(),
                          onSeekingEnded: (value) => controls.onSeekingEnded(value),
                        ),
                      ),
                      Padding(
                        padding: EdgeInsets.only(left: 10),
                        child: Text(
                          counterLabel,
                          style: TextStyle(
                            fontSize: 14,
                            color: Color.fromRGBO(255, 255, 255, 0.7),
                          ),
                        ),
                      ),
                    ],
                  ),
                ],
              ),
            )
          ],
        ),
      ),
    );
  }
}

class TivioPlayerUi extends StatefulWidget {
  final TivioPlayerUiPressControls controls;
  final BetterPlayerController controller;

  StreamController<BetterPlayerEvent> eventPlayerStream;
  TivioPlayerVodSource source;

  TivioPlayerUi({
    @required this.controls,
    @required this.controller,
    @required this.eventPlayerStream,
    @required this.source,
  });

  @override
  createState() => _TivioPlayerUi();
}

class _TivioPlayerUi extends State<TivioPlayerUi> {
  Duration progress = Duration();
  StreamSubscription subscription;
  bool isPlaying = true;
  bool isInFullscreen = true;

  @override
  void initState() {
    super.initState();

    if (subscription != null) {
      subscription.cancel();
    }

    subscription = widget.eventPlayerStream.stream.listen((event) {
      switch (event.betterPlayerEventType) {
        case BetterPlayerEventType.play:
          setState(() {
            isPlaying = true;
          });
          break;
        case BetterPlayerEventType.pause:
          setState(() {
            isPlaying = false;
          });
          break;
        case BetterPlayerEventType.progress:
          setState(() {
            progress = event.parameters['progress'];
          });
          break;
        case BetterPlayerEventType.finished:
          // fix missing last tick
          setState(() {
            progress = widget.controller.videoPlayerController.value.duration;
          });
          break;
        case BetterPlayerEventType.hideFullscreen:
          setState(() {
            isInFullscreen = false;
          });
          break;
        case BetterPlayerEventType.openFullscreen:
          setState(() {
            isInFullscreen = true;
          });
          break;
        default:
          break;
      }
    });
  }

  @override
  Widget build(BuildContext context) {
    return TivioPlayerBasicUi(
      controls: widget.controls,
      isPlaying: isPlaying,
      progress: progress,
      duration: widget.controller.videoPlayerController.value.duration,
      isInFullscreen: isInFullscreen,
    );
  }

  @override
  void dispose() {
    if (subscription != null) {
      subscription.cancel();
    }
    super.dispose();
  }
}
